//---------------------------------------------------------------------------//
//
//	Secret Garden - Main
//
//	Date : 2008/05/02
//
//---------------------------------------------------------------------------//

#include "main.h"
#include "Title.h"
#include "CharacterSelect.h"
#include "GameMain.h"
#include "GameResult.h"
#include "GameMenu.h"
#include "Staff.h"

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	CSecretGardenConfig Config;
	_tcscpy(Config.m_szAppName, _T("Secret Garden"));
	_tcscpy(Config.m_szClassName, _T("_SECRET_GARDEN_CLASS_"));
	Config.m_nWidth = 800;
	Config.m_nHeight = 600;
	Config.m_bFullScreen = false;
	Config.m_nFPS = 60;

	CSecretGarden App;
	App.Init(&Config);
	App.Initialize();
	App.Run();
	App.FinalRelease();
	App.Release();

	return true;
}

//---------------------------------------------------------------------------//
//	Base State-Translator Class
//---------------------------------------------------------------------------//

CGameStateTranslator::CGameStateTranslator()
{
	m_nPrevState = CGameState::NONE;
	m_nNextState = CGameState::NONE;
	m_bTranslate = false;
}

CGameStateTranslator::~CGameStateTranslator()
{
}

//!  · ̽ѵ ȴٰ Ѵ.
bool CGameStateTranslator::Translate()
{
	m_bTranslate = true;

	return m_bTranslate;
}

//!  ѵ Ǵ θ ˾ƿ´.
bool CGameStateTranslator::IsTranslate()
{
	return m_bTranslate;
}

//---------------------------------------------------------------------------//
//	Base Game State Class
//---------------------------------------------------------------------------//

CGameState::CGameState()
: m_pTranslator(NULL)
{
}

CGameState::~CGameState()
{
}

bool CGameState::Init(CGameStateTranslator* pTranslator)
{
	return true;
}

void CGameState::Release()
{
}

int CGameState::Update()
{
	return true;
}

void CGameState::Render(CRenderer* pRenderer)
{
}

CGameStateTranslator* CGameState::GetTranslator()
{
	return m_pTranslator;
}

//---------------------------------------------------------------------------//
//	Secret Garden Class
//---------------------------------------------------------------------------//

CSecretGarden::CSecretGarden() : m_pState(NULL)
{
}
	
CSecretGarden::~CSecretGarden()
{
}

bool CSecretGarden::Init(CFrameworkConfig* pConfig)
{
	CFramework::Init(pConfig);
	
	SetState(CGameState::TITLE, NULL); //# ó ۵ ¸ Ͽ ڽ κ  ۾Ѵ.

	return true;
}

void CSecretGarden::Release()
{
	SAFE_DELETE(m_pState);

	CFramework::Release();
}

//! ҽ εѴ.
bool CSecretGarden::Initialize()
{
	return true;
}

//! ε ҽ Ѵ.
void CSecretGarden::FinalRelease()
{
}

//! 1ʸ  Ӹŭ Ʈ Ѵ.
int CSecretGarden::Update()
{
	if (m_pState)
	{
		m_pState->Update();
		CGameStateTranslator* pTranslator = m_pState->GetTranslator();
		if (pTranslator && pTranslator->IsTranslate())
		{
			SAFE_DELETE(m_pState);
			SetState(pTranslator->m_nNextState, pTranslator);
			SAFE_DELETE(pTranslator);
		}
	}

	return true;
}

//! Ʈ ͸ ȭ鿡 Ѵ. Ʈ ϰ  ð  ȴ.
void CSecretGarden::Render()
{
	m_pRenderer->ClearBuffer(true, 0x00000000, true);

	m_pRenderer->BeginScene();

	if (m_pState)
	{
		m_pState->Render(m_pRenderer);
	}

	m_pRenderer->EndScene();

	static TCHAR szBuff[256];
	_stprintf(szBuff, _T("fps : %4.2f"), m_fRenderFps, m_pTimerSync->GetFps());
	m_pRenderer->DrawTextDC(15, 15, szBuff, RGB(255, 255, 255));
	
	m_pRenderer->Present();
}

void CSecretGarden::SetState(int nGameState, CGameStateTranslator* pTranslator)
{
	SAFE_DELETE(m_pState);

	switch(nGameState)
	{
	case CGameState::EXIT:
		{
			m_bLoop = false;
		}break;

	case CGameState::TITLE:
		{
			m_pState = new CTitle;
			m_pState->Init(pTranslator);
		}break;

	case CGameState::SELECT_CHARACTER:
		{
			m_pState = new CCharacterSelect;
			m_pState->Init(pTranslator);
		}break;

	case CGameState::GAME_MAIN:
		{
			m_pState = new CGameMain;
			m_pState->Init(pTranslator);
		}break;

	case CGameState::GAME_RESULT:
		{
			m_pState = new CGameResult;
			m_pState->Init(pTranslator);
		}break;

	case CGameState::GAME_MENU:
		{
			m_pState = new CGameMenu;
			m_pState->Init(pTranslator);
		}break;

	case CGameState::STAFF:
		{
			m_pState = new CStaff;
			m_pState->Init(pTranslator);
		}break;

	default: ASSERT(0); break;
	}
}