//---------------------------------------------------------------------------//
//
//	Core - Timer
//
//	created : 2008/04/11
//
//---------------------------------------------------------------------------//

#include "core_std.h"
#include "timer.h"

//---------------------------------------------------------------------------//
//	Timer Class
//---------------------------------------------------------------------------//

CTimer::CTimer()
: m_pTimer(NULL)
{
	m_fElapsedTime = 0.0f;
	m_fFps = 0.0f;
	m_fFpsCnt = 0.0f;
	m_fPeriodCnt = 0.0f;
}

CTimer::~CTimer()
{
	SAFE_DELETE(m_pTimer);
}

//! Ÿ̸Ӹ ŸԿ  ʱȭ Ѵ.
bool CTimer::Init(eTimerType eType)
{
	switch(eType)
	{
	case TYPE_TIME_GET_TIME:
		{
			m_pTimer = new CTimeGetTimer;
		}break;

	case TYPE_GET_TICK_COUNT:
		{
			m_pTimer = new CGetTickTimer;
		}break;

	case TYPE_HIGH_RESOLUTION:
		{
			m_pTimer = new CHighResTimer;
		}break;

	default:
		{
			m_pTimer = new CHighResTimer;
		}break;
	}

	if (m_pTimer->Init() == false)
	{
		return false;
	}

	return true;
}

void CTimer::Reset()
{
	m_pTimer->Init();
}

//! Ÿ̸ ν elapsed Time Ѵ.
void CTimer::Update()
{
	m_fFpsCnt += 1.0f;
	m_fElapsedTime = m_pTimer->Update();

	// frame per second
	m_fPeriodCnt += m_fElapsedTime;
	if (m_fPeriodCnt >= 1.0f)
	{
		m_fFps = m_fFpsCnt / m_fPeriodCnt;
		m_fFpsCnt = 0.0f;
		m_fPeriodCnt = 0.0f;
	}
}

//---------------------------------------------------------------------------//
//	TimeGetTime	Timer
//---------------------------------------------------------------------------//

bool CTimeGetTimer::Init()
{
	m_dwCurTime = ::timeGetTime();
	m_dwLastTime = ::timeGetTime();
	return true;
}

float CTimeGetTimer::Update()
{
	::timeBeginPeriod(1); // Ÿ̸ е 

	m_dwCurTime = ::timeGetTime();
	m_fElapsedTime = (float)(m_dwCurTime - m_dwLastTime) / 1000.0f;
	m_dwLastTime = m_dwCurTime;

	::timeEndPeriod(1);

	return m_fElapsedTime;
}

//---------------------------------------------------------------------------//
//	GetTickCount Timer
//---------------------------------------------------------------------------//

bool CGetTickTimer::Init()
{
	m_dwCurTime = ::GetTickCount();
	m_dwLastTime = ::GetTickCount();
	return true;
}

float CGetTickTimer::Update()
{
	::timeBeginPeriod(1); // Ÿ̸ е 

	m_dwCurTime = ::GetTickCount();
	m_fElapsedTime = (float)(m_dwCurTime - m_dwLastTime) / 1000.0f;
	m_dwLastTime = m_dwCurTime;

	::timeEndPeriod(1);

	return m_fElapsedTime;
}

//---------------------------------------------------------------------------//
//	High Resolution Timer (QueryPerformanceCounter)
//---------------------------------------------------------------------------//

bool CHighResTimer::Init()
{
	::QueryPerformanceFrequency(&m_Frequency);
	if (m_Frequency.QuadPart == 0)
	{
		return false;
	}

	::QueryPerformanceCounter(&m_CurTick);
	::QueryPerformanceCounter(&m_LastTick);

	return true;
}

float CHighResTimer::Update()
{
	::QueryPerformanceCounter(&m_CurTick);
	m_fElapsedTime = (float)(m_CurTick.QuadPart - m_LastTick.QuadPart) / (float)m_Frequency.QuadPart;
	m_LastTick.QuadPart = m_CurTick.QuadPart;

	return m_fElapsedTime;
}