//---------------------------------------------------------------------------//
//
//	Framework
//
//	Date : 2008/04/07
//
//---------------------------------------------------------------------------//

#include "framework_std.h"
#include "framework.h"

//---------------------------------------------------------------------------//
//	Framework Configuration Class
//---------------------------------------------------------------------------//

CFrameworkConfig::CFrameworkConfig()
{
}

CFrameworkConfig::~CFrameworkConfig()
{
}

bool CFrameworkConfig::Load(const tstring& strFilename)
{
	return true;
}

//---------------------------------------------------------------------------//
//	Framework Class
//---------------------------------------------------------------------------//

CFramework::CFramework()
: m_pConfig(NULL), m_pTimer(NULL), m_pTimerSync(NULL), m_pRenderer(NULL)
{
	m_fRenderFps = 0.0f;
}

CFramework::~CFramework()
{
}

//! CFrameworkConfig ̿Ͽ ӿũ ʱȭ Ѵ.
bool CFramework::Init(CFrameworkConfig* pConfig)
{
	ASSERT(m_pConfig == NULL && pConfig);
	m_pConfig = pConfig;

	Init();

	return true;
}

bool CFramework::Init()
{
	// window initialization
	if (m_pConfig->m_bFullScreen)
	{
		if (Create(m_pConfig->m_szAppName, m_pConfig->m_szClassName, 0, 0, m_pConfig->m_nWidth, m_pConfig->m_nHeight, 
			WS_POPUP | WS_CLIPCHILDREN, 0, 0, 0) == false)
		{
			return false;
		}
	}
	else
	{
		if (Create(m_pConfig->m_szAppName, m_pConfig->m_szClassName, 0, 0, m_pConfig->m_nWidth, m_pConfig->m_nHeight, 
			WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_CLIPCHILDREN, 0, 0, 0) == false)
		{	
			return false;
		}
	}

	// graphics initialization
	m_pRenderer = GraphicsInitialize(m_hWnd, m_pConfig->m_nWidth, m_pConfig->m_nHeight, m_pConfig->m_bFullScreen);

	// timer initialization
	m_pTimer = new CTimer;
	m_pTimer->Init(CTimer::TYPE_HIGH_RESOLUTION);

	m_pTimerSync = new CTimerSync;
	m_pTimerSync->SetFps(m_pConfig->m_nFPS);

	// input initialization
	GetInputSystem().Init(m_hWnd, GetModuleHandle(NULL), false, DI_USE_KEYBOARD | DI_USE_MOUSE);

	return true;
}

void CFramework::Release()
{
	GetInputSystem().DeleteInstance();

	SAFE_DELETE(m_pTimer);
	SAFE_DELETE(m_pTimerSync);

	GraphicsRelease(m_pRenderer);
}

//! ӿũ η Ѵ. ø̼  ۽Ų.
void CFramework::Run()
{
	m_bLoop = true;
	SetIdle(true);

	MsgLoop();
}

//!  ޽ óϰ  ð Ǵ κ ø̼  Ʈ  Ѵ.
void CFramework::Idle()
{
	m_pTimer->Update();

	if (m_pTimerSync->Update(m_pTimer->GetElapsedTime()) == true)
	{
		GetInputSystem().Update();
		Update();
	}

	//   Ѵ.
	static float fRenderFpsPeriod = 0.0f, fRenderFpsCount = 0.0f;
	fRenderFpsPeriod += m_pTimer->GetElapsedTime();
	if (fRenderFpsPeriod >= 1.0f)
	{
		m_fRenderFps = fRenderFpsCount / fRenderFpsPeriod;
		fRenderFpsCount = 0.0f;
		fRenderFpsPeriod = 0.0f;
	}

	if (m_pTimerSync->IsFrameSkip() == false) //  ŵ ƴϸ  Ѵ.
	{
		if (m_hWnd == GetForegroundWindow())
		{
			/*HRESULT hr = m_pRenderer->GetDevice()->TestCooperativeLevel();
			if (hr == D3DERR_DEVICELOST) // ̽ ҽǵǾ   Ѵ.
			{
			}
			else if (hr == D3DERR_DEVICENOTRESET) // ̽ ҽǵǾ  ɼ ִ.
			{
				m_pRenderer->RestoreDevice();
			}
			else if (hr == D3DERR_DRIVERINTERNALERROR)
			{
			}
			else*/
			{
				Render();
				fRenderFpsCount += 1.0f;
				Sleep(1);
			}
		}
	}
}

//! ӿũ  ν (override)
LRESULT CFramework::WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	return CWindowApp::WndProc(hWnd, Message, wParam, lParam);
}