//---------------------------------------------------------------------------//
//
//	Framework - Input
//
//	created : 2008/04/17
//
//---------------------------------------------------------------------------//

#include "framework_std.h"
#include <stdio.h>
#include "input.h"

CInput& GetInputSystem()
{
	return CInput::GetInstance();
}

CInput::CInput() 
: m_pDI(NULL), m_pKeyboard(NULL), m_pMouse(NULL)
{
	memset(m_szErrorMsg, 0, sizeof(m_szErrorMsg));
}

CInput::~CInput()
{
	Release();
}

//! ̷Ʈ ǲ ü ʱȭ ϰ flags  ġ ʱȭѴ.
bool CInput::Init(HWND hWnd, HINSTANCE hInstance, bool bExclusive, DWORD flags)
{
	m_hWnd = hWnd;

	if (FAILED(DirectInput8Create(hInstance, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&m_pDI, NULL)))
	{
		sprintf(m_szErrorMsg, "DirectInput8 ü  (DirectInput8Create)");
		return false;
	}

	if (flags & DI_USE_KEYBOARD)
	{
		m_pKeyboard = new CKeyboard;
		m_pKeyboard->CreateDevice(m_pDI, hWnd, bExclusive, m_szErrorMsg);
	}
	if (flags & DI_USE_MOUSE)
	{
		m_pMouse = new CMouse;
		m_pMouse->CreateDevice(m_pDI, hWnd, bExclusive, m_szErrorMsg);
	}

	return true;
}

//! ̷Ʈ ǲ ü Ѵ..
void CInput::Release()
{
	SAFE_DELETE(m_pKeyboard);
	SAFE_DELETE(m_pMouse);

	SAFE_RELEASE(m_pDI);
}

//! Է ġ ¸ ĵѴ.
void CInput::Update()
{
	if (m_pKeyboard)
	{
		m_pKeyboard->Update();
	}
	if (m_pMouse)
	{
		m_pMouse->Update(m_hWnd);
	}
}

//-----------------------------------------------------------------------------
//	Keyboard Class
//-----------------------------------------------------------------------------

CKeyboard::CKeyboard() 
: m_pDevice(NULL)
{
	m_WaitInput = 0;
	memset(m_keyStates, 0, sizeof(BYTE)*256);
	memset(m_dwKeyPressTime, 0, sizeof(DWORD)*256);
}

CKeyboard::~CKeyboard()
{
	Release();
}

//! Ű ġ(device)  Ȯ(acquire)Ѵ.
bool CKeyboard::CreateDevice(LPDIRECTINPUT8 lpDI, HWND hWnd, bool bExclusive, char* szErrorMsg)
{
	DWORD flags=0;

	if (FAILED(lpDI->CreateDevice(GUID_SysKeyboard, &m_pDevice, NULL)))
	{
		sprintf(szErrorMsg, "DirectInput8 Ű ġ  (CreateDevice)");
		return false;
	}

	if (FAILED(m_pDevice->SetDataFormat(&c_dfDIKeyboard)))
	{
		sprintf(szErrorMsg, "DirectInput8 Ű Ÿ   (SetDataFormat)");
		return false;
	}

	if (bExclusive)
		flags = DISCL_FOREGROUND | DISCL_EXCLUSIVE | DISCL_NOWINKEY;
	else
		flags = DISCL_FOREGROUND | DISCL_NONEXCLUSIVE;

	if (FAILED(m_pDevice->SetCooperativeLevel(hWnd, flags)))
	{
		sprintf(szErrorMsg, "DirectInput8 Ű ·  (SetCooperativeLevel)");
		return false;
	}

	if (FAILED(m_pDevice->Acquire()))
	{
		sprintf(szErrorMsg, "DirectInput8 Ű ġ Ȯ (Acquire)");
		return false;
	}
	return true;
}

//! Ű ġ Ѵ.
void CKeyboard::Release()
{
	if (m_pDevice)
	{
		m_pDevice->Unacquire();
		m_pDevice->Release();
		m_pDevice = NULL;
	}
}

//! Ű ¸ üũϰ Ʈ Ѵ.
bool CKeyboard::Update()
{
	BYTE keyStates[256];
	if (FAILED(m_pDevice->GetDeviceState(sizeof(keyStates), (LPVOID)keyStates)))
	{
		m_pDevice->Acquire();
		return false;
	}
	m_KeyEvent = false;
	m_WaitInput++;
	for (int i=0; i<256; i++)
	{
		if (keyStates[i] & 0x80) // Ű  ̸
		{
			if (m_keyStates[i] == DIK_NONE || m_keyStates[i] == DIK_RELEASE)
			{
				m_keyStates[i] = DIK_PRESS;
				m_dwKeyPressTime[i] = 1;
				m_KeyEvent = true;
				m_WaitInput = 0;
			}
			else if (m_keyStates[i] == DIK_PRESS)
			{
				m_keyStates[i] = DIK_REPEAT;
				m_dwKeyPressTime[i] = 2;
			}
			else
			{
				m_dwKeyPressTime[i]++;
			}
		}
		else
		{
			if (m_keyStates[i] == DIK_PRESS || m_keyStates[i] == DIK_REPEAT)
			{
				m_keyStates[i] = DIK_RELEASE;
				m_KeyEvent = true;
				m_WaitInput = 0;
			}
			else if (m_keyStates[i] == DIK_RELEASE)
			{
				m_keyStates[i] = DIK_NONE;
				m_dwKeyPressTime[i] = 0;
			}
		}
	}
	return true;
}

//-----------------------------------------------------------------------------
//	Mouse Class
//-----------------------------------------------------------------------------

CMouse::CMouse() 
: m_pDevice(NULL)
{
	memset(&m_states, 0, sizeof(DIMOUSESTATE));
}

CMouse::~CMouse()
{
	Release();
}

//! 콺 ġ(device)  Ȯ(acquire)Ѵ.
bool CMouse::CreateDevice(LPDIRECTINPUT8 lpDI, HWND hWnd, bool bExclusive, char *szErrorMsg)
{
	DWORD flags=0;

	if (FAILED(lpDI->CreateDevice(GUID_SysMouse, &m_pDevice, NULL)))
	{
		sprintf(szErrorMsg, "DirectInput8 콺 ġ  (CreateDevice)");
		return false;
	}

	if (FAILED(m_pDevice->SetDataFormat(&c_dfDIMouse)))
	{
		sprintf(szErrorMsg, "DirectInput8 콺 Ÿ   (SetDataFormat)");
		return false;
	}

	if (bExclusive)
		flags = DISCL_FOREGROUND | DISCL_EXCLUSIVE | DISCL_NOWINKEY;
	else
		flags = DISCL_FOREGROUND | DISCL_NONEXCLUSIVE;

	if (FAILED(m_pDevice->SetCooperativeLevel(hWnd, flags)))
	{
		sprintf(szErrorMsg, "DirectInput8 콺 ·  (SetCooperativeLevel)");
		return false;
	}

	if (FAILED(m_pDevice->Acquire()))
	{
		sprintf(szErrorMsg, "DirectInput8 콺 ġ Ȯ (Acquire)");
		return false;
	}
	return true;
}

//! 콺 ġ Ѵ.
void CMouse::Release()
{
	if (m_pDevice)
	{
		m_pDevice->Unacquire();
		m_pDevice->Release();
		m_pDevice = NULL;
	}
}

//! 콺  ¸ ˾ƿ´. (ư , ̵  ǥ, 콺  ǥ)
bool CMouse::Update(HWND hWnd)
{
	DIMOUSESTATE states;

	if (FAILED(m_pDevice->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&states)))
	{	
		m_pDevice->Acquire();
		return false;
	}
	
	m_MouseEvent = false;
	m_PrevMousePos = m_MousePos;
	GetMousePos(hWnd);

	if (m_PrevMousePos.x != m_MousePos.x || m_PrevMousePos.y != m_MousePos.y) 
	{
		m_MouseEvent = true;
	}

	// 콺 ̵  Ÿ(z )
	m_states.lX = states.lX;
	m_states.lY = states.lY;
	m_states.lZ = states.lZ;

	for (int i=0; i<4; i++)
	{
		if (states.rgbButtons[i] & 0x80) // 콺 ư  ̸
		{
			if (m_states.rgbButtons[i] == DIM_NONE || m_states.rgbButtons[i] == DIM_RELEASE)
			{
				m_states.rgbButtons[i] = DIM_PRESS;
				m_MouseEvent = true;
			}
			else if (m_states.rgbButtons[i] == DIM_PRESS)
			{
				m_states.rgbButtons[i] = DIM_REPEAT;
			}
		}
		else
		{
			if (m_states.rgbButtons[i] == DIM_PRESS || m_states.rgbButtons[i] == DIM_REPEAT)
			{
				m_states.rgbButtons[i] = DIM_RELEASE;
				m_MouseEvent = true;
			}
			else if (m_states.rgbButtons[i] == DIM_RELEASE)
			{
				m_states.rgbButtons[i] = DIM_NONE;
			}
		}
	}
	return true;
}
