//---------------------------------------------------------------------------//
//
//	Graphics - Mathematics
//
//	Date : 2008/04/07
//
//---------------------------------------------------------------------------//

#pragma once

#include <d3dx9math.h>

typedef D3DXVECTOR2		vector2;
typedef D3DXVECTOR3		vector3;
typedef D3DXVECTOR4		vector4;
typedef D3DXMATRIX		matrix4;
typedef D3DXQUATERNION	quaternion;
typedef D3DXPLANE		plane;

//!  3 ͸ Ѵ
GRAPHICS_API inline const vector3& vec3_add(vector3& vOut, const vector3& v1, const vector3& v2)
{
	return *(D3DXVec3Add(&vOut, &v1, &v2));
}

//!  3 ͸ .
GRAPHICS_API inline const vector3& vec3_subtract(vector3& vOut, const vector3& v1, const vector3& v2)
{
	return *(D3DXVec3Subtract(&vOut, &v1, &v2));
}

//!  3 ͸ Ѵ.
GRAPHICS_API inline float vec3_dot(const vector3& v1, const vector3& v2)
{
	return D3DXVec3Dot(&v1, &v2);
}

//!  3 ͸ Ѵ.
GRAPHICS_API inline const vector3& vec3_cross(vector3& vOut, const vector3& v1, const vector3& v2)
{
	return *(D3DXVec3Cross(&vOut, &v1, &v2));
}

//! 3  ̸ Ѵ.
GRAPHICS_API inline float vec3_length(const vector3& v1)
{
	return D3DXVec3Length(&v1);
}

//!  3 ͸ t    ͸ Ѵ. v1 + t(v2 - v1)
GRAPHICS_API inline const vector3& vec3_lerp(vector3& vOut, const vector3& v1, const vector3& v2, float t)
{
	return *(D3DXVec3Lerp(&vOut, &v1, &v2, t));
}

//! 3 ǥ 2 ǥ ȯش.
GRAPHICS_API inline const vector3& vec3_3dTo2d(vector3& vOut, const vector3& v1, const D3DVIEWPORT9 &Viewport,
									const matrix4 &matProj, const matrix4 &matView, const matrix4 &matWorld)
{
	return *(D3DXVec3Project(&vOut, &v1, &Viewport, &matProj, &matView, &matWorld));
}

//! 2 ǥ 3 ǥ ȯش.
GRAPHICS_API inline const vector3& vec3_2dTo3d(vector3& vOut, const vector3& v1, const D3DVIEWPORT9 &Viewport,
									const matrix4 &matProj, const matrix4 &matView, const matrix4 &matWorld)
{
	return *(D3DXVec3Unproject(&vOut, &v1, &Viewport, &matProj, &matView, &matWorld));
}

//! 3  ְ Ѵ.
GRAPHICS_API inline const vector3& vec3_normalize(vector3& vOut, const vector3& v1)
{
	return *(D3DXVec3Normalize(&vOut, &v1));
}

//! 3 ͸   ̿Ͽ ȯѴ. 
GRAPHICS_API inline const vector3& vec3_transform(vector3& vOut, const vector3& v1, const matrix4& m1)
{
	return *(D3DXVec3TransformCoord(&vOut, &v1, &m1));
}

//! 4x4  ķ ʱȭ Ѵ.
GRAPHICS_API inline const matrix4& mat4_identity(matrix4& m1)
{
	return *(D3DXMatrixIdentity(&m1));
}

GRAPHICS_API inline vector3 operator*(const vector3& V1, const vector3& V2)
{
	return vector3( V1.x * V2.x, V1.y * V2.y, V1.z * V2.z );
}

//! 4x4   Ѵ.
/*!
	@param pmOut [out]     ϴµ  NULL Ѵ.
	@param m1 [in]  Ϸ 
	@param pfDeterminant [out] Ǻ, ʿ  NULL (Ʈ ڷ NULL)
	@return     ϴµ  NULL Ѵ.
*/
GRAPHICS_API inline const matrix4* mat4_inverse(matrix4* pmOut, const matrix4& m1, float* pfDeterminant=NULL)
{
	return D3DXMatrixInverse(pmOut, pfDeterminant, &m1);
}

//!  4x4  Ѵ.
GRAPHICS_API inline const matrix4& mat4_multiply(matrix4& mOut, const matrix4& m1, const matrix4& m2)
{
	return *(D3DXMatrixMultiply(&mOut, &m1, &m2));
}

//! 4x4  ġ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_transpose(matrix4& mOut, const matrix4& m1)
{
	return *(D3DXMatrixTranspose(&mOut, &m1));
}

//!   ޾ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_axis(matrix4& mOut, const vector3& vecAxis, float fAngle)
{
	return *(D3DXMatrixRotationAxis(&mOut, &vecAxis, fAngle));
}

//!  ޾ x  ȸ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_x(matrix4& mOut, float fAngle)
{
	return *(D3DXMatrixRotationX(&mOut, fAngle));
}

//!  ޾ y  ȸ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_y(matrix4& mOut, float fAngle)
{
	return *(D3DXMatrixRotationY(&mOut, fAngle));
}

//!  ޾ z  ȸ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_z(matrix4& mOut, float fAngle)
{
	return *(D3DXMatrixRotationZ(&mOut, fAngle));
}

//! yaw, pitch, roll  ޾ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_yawpitchroll(matrix4& mOut, float yaw, float pitch, float roll)
{
	return *(D3DXMatrixRotationYawPitchRoll(&mOut, yaw, pitch, roll));
}

//! ʹϿ ȸ  Ѵ.
GRAPHICS_API inline const matrix4& mat4_rot_quaternion(matrix4& mOut, const quaternion& q1)
{
	return *(D3DXMatrixRotationQuaternion(&mOut, &q1));
}

//! ʹϿ  ʹϿ ʱȭ Ѵ.
GRAPHICS_API inline const quaternion& qua_identity(quaternion& qOut)
{
	return *(D3DXQuaternionIdentity(&qOut));
}

//!  ʹϿ Ѵ. q1 * q2
GRAPHICS_API inline const quaternion& qua_multiply(quaternion& qOut, const quaternion& q1, const quaternion& q2)
{
	return *(D3DXQuaternionMultiply(&qOut, &q1, &q2));
}

//!   ʹϿ Ѵ.
GRAPHICS_API inline const quaternion& qua_rot_axis(quaternion& qOut, const vector3& vAxis, float fAngle)
{
	return *(D3DXQuaternionRotationAxis(&qOut, &vAxis, fAngle));
}

//! ķ ʹϿ Ѵ.
GRAPHICS_API inline const quaternion& qua_rot_matrix(quaternion& qOut, const matrix4& m1)
{
	return *(D3DXQuaternionRotationMatrix(&qOut, &m1));
}

//!  ʹϿ 鼱 Ѵ.
GRAPHICS_API inline const quaternion& qua_slerp(quaternion& qOut, const quaternion& q1, const quaternion& q2, float t)
{
	return *(D3DXQuaternionSlerp(&qOut, &q1, &q2, t));
}

//!   (a,b,c)  ֶ Ѵ.
GRAPHICS_API inline const plane& plane_normalize(plane& ResultPlane, const plane& SrcPlane)
{
	return *(D3DXPlaneNormalize(&ResultPlane, &SrcPlane));
}

//!  3 ͸ Ѵ. (ax + by + cz + d)
GRAPHICS_API inline float plane_dot_coord(const plane& SrcPlane, const vector3& vCoord)
{
	return D3DXPlaneDotCoord(&SrcPlane, &vCoord);
}

//!  3  ͸ Ѵ. (ax + by + cz)
GRAPHICS_API inline float plane_dot_normal(const plane& SrcPlane, const vector3& vNormal)
{
	return D3DXPlaneDotNormal(&SrcPlane, &vNormal);
}

//!  ȯ ķ ȯѴ.
GRAPHICS_API inline const plane& plane_transform(plane& ResultPlane, const plane& SrcPlane, const matrix4& matTransform)
{
	return *(D3DXPlaneTransform(&ResultPlane, &SrcPlane, &matTransform));
}